@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2017-2022 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2017-2023 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -23,11 +23,13 @@
 #include <wx/filesys.h>
 #include <wx/fontutil.h>
 #include <wx/fs_zip.h>
+#include <wx/log.h>
 #include <wx/stdpaths.h>
 
 #include "App.hpp"
 #include "ArtProvider.hpp"
 #include "DiffWindow.hpp"
+#include "IPC.hpp"
 #include "mainwindow.hpp"
 #include "Palette.hpp"
 #include "profile.hpp"
@@ -43,6 +45,7 @@ IMPLEMENT_APP(REHex::App);
 bool REHex::App::OnInit()
 {
 	bulk_updates_freeze_count = 0;
+	quick_exit = false;
 	
 	#ifdef BUILD_HELP
 	help_controller = NULL;
@@ -51,6 +54,7 @@ bool REHex::App::OnInit()
 	
 	locale = new wxLocale(wxLANGUAGE_DEFAULT);
 	console = new ConsoleBuffer();
+	thread_pool = new ThreadPool(std::thread::hardware_concurrency());
 	
 	bool process_switches = true;
 	bool compare_mode = false;
@@ -84,7 +88,22 @@ bool REHex::App::OnInit()
 			}
 		}
 		
-		open_filenames.push_back(argv[i].ToStdString());
+		/* If the filename ends in .rehex-meta and stripping it off points to an existing
+		 * file, then assume they mean to open that file - the meta file being considered
+		 * like a "project".
+		*/
+		
+		std::string filename = argv[i].ToStdString();
+		std::string meta_extension = ".rehex-meta";
+		
+		if(filename.length() >= meta_extension.length()
+			&& filename.substr(filename.length() - meta_extension.length()) == meta_extension
+			&& wxFileExists(filename.substr(0, filename.length() - meta_extension.length())))
+		{
+			filename = filename.substr(0, filename.length() - meta_extension.length());
+		}
+		
+		open_filenames.push_back(filename);
 	}
 	
 	if(compare_mode && open_filenames.size() < 2)
@@ -94,6 +113,71 @@ bool REHex::App::OnInit()
 		return false;
 	}
 	
+	bool ipc_params_ok = false;
+	std::string ipc_host;
+	std::string ipc_service;
+	std::string ipc_topic;
+	
+	try {
+		ipc_host      = get_ipc_host();
+		ipc_service   = get_ipc_service();
+		ipc_topic     = get_ipc_topic();
+		ipc_params_ok = true;
+	}
+	catch(const std::exception &e)
+	{
+		fprintf(stderr, "Unable to get IPC parameters: %s\n", e.what());
+	}
+	
+	if(ipc_params_ok)
+	{
+		/* wxDDEClient logs if it can't connect, which comes out as a messagebox. Failing
+		 * to connect is normal if we are the first instance of rehex, so just turn off
+		 * logging while we try to connect to an existing instance...
+		*/
+		wxLogNull bequiet;
+		
+		IPCClient ipc_client;
+		
+		wxConnectionBase *ipc = ipc_client.MakeConnection(ipc_host, ipc_service, ipc_topic);
+		if(ipc != NULL)
+		{
+			quick_exit = true;
+			quick_exit_code = 0;
+			
+			if(compare_mode)
+			{
+				std::vector<std::string> command = { "compare" };
+				
+				for(auto filename = open_filenames.begin(); filename != open_filenames.end(); ++filename)
+				{
+					command.push_back(*filename);
+				}
+				
+				std::string encoded_command = encode_command(command);
+				bool ok = ipc->Execute(encoded_command);
+				
+				if(!ok)
+				{
+					quick_exit_code = 1;
+				}
+			}
+			else{
+				for(auto filename = open_filenames.begin(); filename != open_filenames.end(); ++filename)
+				{
+					std::vector<std::string> command = { "open", *filename };
+					std::string encoded_command = encode_command(command);
+					
+					ipc->Execute(encoded_command);
+				}
+			}
+			
+			ipc->Disconnect();
+			
+			return true;
+		}
+	}
+	
 	call_setup_hooks(SetupPhase::EARLY);
 	
 	#ifdef _WIN32
@@ -109,6 +193,8 @@ bool REHex::App::OnInit()
 	config = new wxConfig("REHex");
 	config->SetPath("/");
 	
+	wxConfig::Set(config);
+	
 	settings = new AppSettings(config);
 	
 	last_directory = config->Read("last-directory", "");
@@ -163,6 +249,8 @@ bool REHex::App::OnInit()
 		active_palette = Palette::create_system_palette();
 	}
 	
+	Bind(EVT_PAGE_DROPPED, &REHex::App::OnTabDropped, this);
+	
 	call_setup_hooks(SetupPhase::READY);
 	
 	wxSize windowSize(740, 540);
@@ -182,13 +270,8 @@ bool REHex::App::OnInit()
 	if(compare_mode)
 	{
 		DiffWindow::instance = new DiffWindow(NULL);
+		DiffWindow::instance->set_invisible_owner_window(window);
 		DiffWindow::instance->Show(true);
-		
-		/* Special hacky handlers to deal with DiffWindow being the only visible window...
-		 * see the comments in them.
-		*/
-		window->Bind(wxEVT_SHOW, &REHex::App::OnMainWindowShow, this);
-		DiffWindow::instance->Bind(wxEVT_CLOSE_WINDOW, &REHex::App::OnDiffWindowClose, this);
 	}
 	else{
 		window->Show();
@@ -222,6 +305,23 @@ bool REHex::App::OnInit()
 		window->new_file();
 	}
 	
+	if(ipc_params_ok)
+	{
+		ipc_server = new IPCServer;
+		bool ipc_ok = ipc_server->Create(ipc_service);
+		
+		if(ipc_ok)
+		{
+			printf_info("IPC service created (%s)\n", ipc_service.c_str());
+		}
+		else{
+			printf_error("Unable to create IPC service (%s)\n", ipc_service.c_str());
+			
+			delete ipc_server;
+			ipc_server = NULL;
+		}
+	}
+	
 	#ifdef REHEX_PROFILE
 	ProfilingWindow *pw = new ProfilingWindow(window);
 	pw->Show();
@@ -234,6 +334,11 @@ bool REHex::App::OnInit()
 
 int REHex::App::OnExit()
 {
+	if(quick_exit)
+	{
+		return 0;
+	}
+	
 	call_setup_hooks(SetupPhase::SHUTDOWN);
 	
 	config->SetPath("/recent-files/");
@@ -244,13 +349,13 @@ int REHex::App::OnExit()
 	
 	settings->write(config);
 	
+	delete ipc_server;
 	delete active_palette;
 	#ifdef BUILD_HELP
 	delete help_controller;
 	#endif
 	delete recent_files;
 	delete settings;
-	delete config;
 	
 	#ifdef _WIN32
 	CoUninitialize();
@@ -258,6 +363,9 @@ int REHex::App::OnExit()
 	
 	call_setup_hooks(SetupPhase::SHUTDOWN_LATE);
 	
+	delete thread_pool;
+	thread_pool = NULL;
+	
 	delete console;
 	console = NULL;
 	
@@ -267,33 +375,31 @@ int REHex::App::OnExit()
 	return 0;
 }
 
-void REHex::App::OnMainWindowShow(wxShowEvent &event)
+int REHex::App::OnRun()
 {
-	/* This handler gets called if the MainWindow is shown because of an action in the
-	 * DiffWindow when the --compare switch was used.
-	 *
-	 * We remove our hacky handlers and let things go as normal now.
-	*/
-	
-	if(event.IsShown())
+	if(quick_exit)
 	{
-		DiffWindow::instance->Unbind(wxEVT_CLOSE_WINDOW, &REHex::App::OnDiffWindowClose, this);
-		window->Unbind(wxEVT_SHOW, &REHex::App::OnMainWindowShow, this);
+		return quick_exit_code;
+	}
+	else{
+		return wxApp::OnRun();
 	}
-	
-	event.Skip();
 }
 
-void REHex::App::OnDiffWindowClose(wxCloseEvent &event)
+void REHex::App::OnTabDropped(DetachedPageEvent &event)
 {
-	/* This handler gets called if the DiffWindow created as the sole visible top-level window
-	 * when using the --compare switch was closed. We destroy the (invisible) MainWindow so the
-	 * program will exit.
+	/* We get triggered by DetachableNotebook when a document tab is detached from a
+	 * MainWindow and then dropped elsewhere, wherein we set up a new MainWindow to own it.
 	*/
 	
-	if(event.GetEventObject() == DiffWindow::instance)
-	{
-		window->Destroy();
-		event.Skip();
-	}
+	wxPoint mouse_position = wxGetMousePosition();
+	
+	Tab *tab = dynamic_cast<Tab*>(event.page);
+	assert(tab != NULL);
+	
+	MainWindow *window = new MainWindow(wxDefaultSize);
+	window->SetClientSize(tab->GetParent()->GetSize());
+	window->SetPosition(mouse_position);
+	window->insert_tab(tab, -1);
+	window->Show();
 }
