@@ -130,7 +130,7 @@ namespace REHex
 			const std::vector<InstructionRange> &get_processed() const { return processed; }
 			
 			virtual int calc_width(DocumentCtrl &doc_ctrl) override;
-			virtual void calc_height(DocumentCtrl &doc_ctrl, wxDC &dc) override;
+			virtual void calc_height(DocumentCtrl &doc_ctrl) override;
 			
 			virtual void draw(DocumentCtrl &doc_ctrl, wxDC &dc, int x, int64_t y) override;
 			
