@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2017-2022 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2017-2023 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -40,8 +40,8 @@
 #include "NumericEntryDialog.hpp"
 #include "NumericTextCtrl.hpp"
 #include "Palette.hpp"
+#include "RangeDialog.hpp"
 #include "search.hpp"
-#include "SelectRangeDialog.hpp"
 #include "SharedDocumentPointer.hpp"
 #include "ToolPanel.hpp"
 #include "util.hpp"
@@ -102,12 +102,14 @@ enum {
 
 BEGIN_EVENT_TABLE(REHex::MainWindow, wxFrame)
 	EVT_CLOSE(REHex::MainWindow::OnWindowClose)
+	EVT_ACTIVATE(REHex::MainWindow::OnWindowActivate)
 	EVT_CHAR_HOOK(REHex::MainWindow::OnCharHook)
 	
 	EVT_MENU(wxID_NEW,        REHex::MainWindow::OnNew)
 	EVT_MENU(wxID_OPEN,       REHex::MainWindow::OnOpen)
 	EVT_MENU(wxID_SAVE,       REHex::MainWindow::OnSave)
 	EVT_MENU(wxID_SAVEAS,     REHex::MainWindow::OnSaveAs)
+	EVT_MENU(wxID_REFRESH,    REHex::MainWindow::OnReload)
 	EVT_MENU(ID_IMPORT_HEX,   REHex::MainWindow::OnImportHex)
 	EVT_MENU(ID_EXPORT_HEX,   REHex::MainWindow::OnExportHex)
 	EVT_MENU(wxID_CLOSE,      REHex::MainWindow::OnClose)
@@ -191,6 +193,7 @@ BEGIN_EVENT_TABLE(REHex::MainWindow, wxFrame)
 	EVT_AUINOTEBOOK_PAGE_CLOSED(   wxID_ANY, REHex::MainWindow::OnDocumentClosed)
 	EVT_AUINOTEBOOK_TAB_RIGHT_DOWN(wxID_ANY, REHex::MainWindow::OnDocumentMenu)
 	EVT_AUINOTEBOOK_TAB_MIDDLE_UP( wxID_ANY, REHex::MainWindow::OnDocumentMiddleMouse)
+	EVT_DETACHABLENOTEBOOK_PAGE_DETACHED(wxID_ANY, REHex::MainWindow::OnDocumentDetached)
 	
 	EVT_CURSORUPDATE(wxID_ANY, REHex::MainWindow::OnCursorUpdate)
 	
@@ -199,8 +202,19 @@ BEGIN_EVENT_TABLE(REHex::MainWindow, wxFrame)
 	EVT_COMMAND(wxID_ANY, REHex::EV_UNDO_UPDATE,       REHex::MainWindow::OnUndoUpdate)
 	EVT_COMMAND(wxID_ANY, REHex::EV_BECAME_DIRTY,      REHex::MainWindow::OnBecameDirty)
 	EVT_COMMAND(wxID_ANY, REHex::EV_BECAME_CLEAN,      REHex::MainWindow::OnBecameClean)
+	EVT_COMMAND(wxID_ANY, REHex::BACKING_FILE_DELETED, REHex::MainWindow::OnFileDeleted)
+	EVT_COMMAND(wxID_ANY, REHex::BACKING_FILE_MODIFIED, REHex::MainWindow::OnFileModified)
+	
+	EVT_DOCUMENTTITLE(wxID_ANY, REHex::MainWindow::OnTitleChanged)
 END_EVENT_TABLE()
 
+std::list<REHex::MainWindow*> REHex::MainWindow::instances;
+
+const std::list<REHex::MainWindow*> &REHex::MainWindow::get_instances()
+{
+	return instances;
+}
+
 REHex::MainWindow::MainWindow(const wxSize& size):
 	wxFrame(NULL, wxID_ANY, "Reverse Engineers' Hex Editor", wxDefaultPosition, size),
 	menu_bar(NULL),
@@ -230,6 +244,10 @@ REHex::MainWindow::MainWindow(const wxSize& size):
 		
 		file_menu->AppendSeparator(); /* ---- */
 		
+		file_menu->Append(wxID_REFRESH, "&Reload");
+		
+		file_menu->AppendSeparator(); /* ---- */
+		
 		file_menu->Append(ID_IMPORT_HEX, "&Import Intel Hex File");
 		file_menu->Append(ID_EXPORT_HEX, "E&xport Intel Hex File");
 		
@@ -511,12 +529,16 @@ REHex::MainWindow::MainWindow(const wxSize& size):
 	
 	toolbar->Realize();
 	
-	notebook = new wxAuiNotebook(this, wxID_ANY, wxDefaultPosition, wxDefaultSize,
+	static int DOCUMENT_PAGE_GROUP;
+	notebook = new DetachableNotebook(this, wxID_ANY, &DOCUMENT_PAGE_GROUP, &(wxGetApp()), wxDefaultPosition, wxDefaultSize,
 		(wxAUI_NB_TOP | wxAUI_NB_TAB_MOVE | wxAUI_NB_SCROLL_BUTTONS | wxAUI_NB_CLOSE_ON_ALL_TABS));
 	
 	notebook_dirty_bitmap = artp.GetBitmap(wxART_FILE_SAVE, wxART_MENU);
 	assert(!notebook_dirty_bitmap.IsSameAs(wxNullBitmap));
 	
+	notebook_bad_bitmap = artp.GetBitmap(wxART_MISSING_IMAGE, wxART_MENU);
+	assert(!notebook_bad_bitmap.IsSameAs(wxNullBitmap));
+	
 	CreateStatusBar(3);
 	
 	SetDropTarget(new DropTarget(this));
@@ -549,12 +571,16 @@ REHex::MainWindow::MainWindow(const wxSize& size):
 	
 	SetIcons(icons);
 	
+	instances.push_back(this);
+	instances_iter = std::prev(instances.end());
+	
 	call_setup_hooks(SetupPhase::DONE);
 }
 
 REHex::MainWindow::~MainWindow()
 {
 	wxGetApp().recent_files->RemoveMenu(recent_files_menu);
+	instances.erase(instances_iter);
 }
 
 void REHex::MainWindow::new_file()
@@ -657,7 +683,21 @@ REHex::Tab *REHex::MainWindow::import_hex_file(const std::string &filename)
 
 void REHex::MainWindow::OnWindowClose(wxCloseEvent &event)
 {
-	if(!unsaved_confirm())
+	std::vector<Tab*> closing_tabs;
+	
+	size_t num_tabs = notebook->GetPageCount();
+	for(size_t i = 0; i < num_tabs; ++i)
+	{
+		wxWindow *page = notebook->GetPage(i);
+		assert(page != NULL);
+		
+		auto p_tab = dynamic_cast<Tab*>(page);
+		assert(p_tab != NULL);
+		
+		closing_tabs.push_back(p_tab);
+	}
+	
+	if(!confirm_close_tabs(closing_tabs))
 	{
 		/* Stop the window from being closed. */
 		event.Veto();
@@ -668,6 +708,19 @@ void REHex::MainWindow::OnWindowClose(wxCloseEvent &event)
 	event.Skip();
 }
 
+void REHex::MainWindow::OnWindowActivate(wxActivateEvent &event)
+{
+	if(event.GetActive())
+	{
+		instances.erase(instances_iter);
+		
+		instances.push_front(this);
+		instances_iter = instances.begin();
+	}
+	
+	event.Skip();
+}
+
 void REHex::MainWindow::OnCharHook(wxKeyEvent &event)
 {
 	int modifiers = event.GetModifiers();
@@ -744,7 +797,6 @@ void REHex::MainWindow::OnSave(wxCommandEvent &event)
 	
 	try {
 		tab->doc->save();
-		_update_dirty(tab->doc);
 	}
 	catch(const std::exception &e)
 	{
@@ -757,40 +809,17 @@ void REHex::MainWindow::OnSave(wxCommandEvent &event)
 
 void REHex::MainWindow::OnSaveAs(wxCommandEvent &event)
 {
-	std::string dir, name;
-	std::string doc_filename = active_document()->get_filename();
-	
-	if(doc_filename != "")
-	{
-		wxFileName wxfn(doc_filename);
-		wxfn.MakeAbsolute();
-		
-		dir  = wxfn.GetPath();
-		name = wxfn.GetFullName();
-	}
-	else{
-		dir  = wxGetApp().get_last_directory();
-		name = "";
-	}
-	
-	wxFileDialog saveFileDialog(this, "Save As", dir, name, "", wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
-	if(saveFileDialog.ShowModal() == wxID_CANCEL)
-		return;
-	
-	std::string filename = saveFileDialog.GetPath().ToStdString();
-	
-	{
-		wxFileName wxfn(filename);
-		wxString dirname = wxfn.GetPath();
-		
-		wxGetApp().set_last_directory(dirname.ToStdString());
-	}
-	
 	Tab *tab = active_tab();
 	
+	std::string filename = document_save_as_dialog(this, tab->doc);
+	if(filename == "")
+	{
+		/* Cancelled. */
+		return;
+	}
+	
 	try {
 		tab->doc->save(filename);
-		_update_dirty(tab->doc);
 	}
 	catch(const std::exception &e)
 	{
@@ -799,8 +828,36 @@ void REHex::MainWindow::OnSaveAs(wxCommandEvent &event)
 			"Error", wxICON_ERROR, this);
 		return;
 	}
+}
+
+void REHex::MainWindow::OnReload(wxCommandEvent &event)
+{
+	Document *doc = active_document();
 	
-	notebook->SetPageText(notebook->GetSelection(), tab->doc->get_title());
+	assert(!doc->get_filename().empty());
+	
+	if(doc->is_dirty())
+	{
+		std::string msg
+			= "The content of " + doc->get_title() + " has been modified.\n"
+			+ "Discard changes and reload file?";
+		
+		int res = wxMessageBox(msg, "File data modified", (wxYES_NO | wxICON_EXCLAMATION), this);
+		if(res == wxNO)
+		{
+			return;
+		}
+	}
+	
+	try {
+		doc->reload();
+	}
+	catch(const std::exception &e)
+	{
+		wxMessageBox(
+			std::string("Error reloading ") + doc->get_title() + ":\n" + e.what(),
+			"Error", wxICON_ERROR, this);
+	}
 }
 
 void REHex::MainWindow::OnImportHex(wxCommandEvent &event)
@@ -1346,8 +1403,29 @@ void REHex::MainWindow::OnSelectRange(wxCommandEvent &event)
 {
 	Tab *tab = active_tab();
 	
-	REHex::SelectRangeDialog srd(this, *(tab->doc), *(tab->doc_ctrl));
-	srd.ShowModal();
+	REHex::RangeDialog rd(this, tab->doc_ctrl, "Select range", true);
+	
+	if(tab->doc_ctrl->has_selection())
+	{
+		off_t selection_first, selection_last;
+		std::tie(selection_first, selection_last) = tab->doc_ctrl->get_selection_raw();
+		
+		rd.set_range_raw(selection_first, selection_last);
+	}
+	else{
+		rd.set_offset_hint(tab->doc_ctrl->get_cursor_position());
+	}
+	
+	int s = rd.ShowModal();
+	if(s == wxID_OK)
+	{
+		assert(rd.range_valid());
+		
+		off_t range_first, range_last;
+		std::tie(range_first, range_last) = rd.get_range_raw();
+		
+		tab->doc_ctrl->set_selection_raw(range_first, range_last);
+	}
 }
 
 void REHex::MainWindow::OnFillRange(wxCommandEvent &event)
@@ -1652,6 +1730,8 @@ void REHex::MainWindow::OnDocumentChange(wxAuiNotebookEvent& event)
 	
 	Tab *tab = active_tab();
 	
+	file_menu->Enable(wxID_REFRESH, !tab->doc->get_filename().empty());
+	
 	edit_menu->Check(ID_OVERWRITE_MODE, !tab->doc_ctrl->get_insert_mode());
 	edit_menu->Check(ID_WRITE_PROTECT, tab->doc->get_write_protect());
 	view_menu->Check(ID_SHOW_OFFSETS, tab->doc_ctrl->get_show_offsets());
@@ -1817,6 +1897,15 @@ void REHex::MainWindow::OnDocumentMiddleMouse(wxAuiNotebookEvent& event)
 	close_tab(tab);
 }
 
+void REHex::MainWindow::OnDocumentDetached(DetachedPageEvent &event)
+{
+	if(notebook->GetPageCount() == 0)
+	{
+		/* Detached the last tab - close the window. */
+		Destroy();
+	}
+}
+
 void REHex::MainWindow::OnCursorUpdate(CursorUpdateEvent &event)
 {
 	Tab *active_tab = this->active_tab();
@@ -1897,38 +1986,55 @@ void REHex::MainWindow::OnUndoUpdate(wxCommandEvent &event)
 
 void REHex::MainWindow::OnBecameDirty(wxCommandEvent &event)
 {
-	wxWindow *cpage = notebook->GetCurrentPage();
-	assert(cpage != NULL);
-	
-	auto tab = dynamic_cast<Tab*>(cpage);
-	assert(tab != NULL);
-	
-	auto doc = dynamic_cast<REHex::Document*>(event.GetEventObject());
-	assert(doc != NULL);
-	
-	if(doc == tab->doc)
-	{
-		/* Only update the window if the event originated from the active document. */
-		_update_dirty(tab->doc);
-	}
+	Document *event_doc = (Document*)(event.GetEventObject());
+	_update_dirty(event_doc);
 }
 
 void REHex::MainWindow::OnBecameClean(wxCommandEvent &event)
 {
-	wxWindow *cpage = notebook->GetCurrentPage();
-	assert(cpage != NULL);
+	Document *event_doc = (Document*)(event.GetEventObject());
+	_update_dirty(event_doc);
+}
+
+void REHex::MainWindow::OnFileDeleted(wxCommandEvent &event)
+{
+	Document *event_doc = (Document*)(event.GetEventObject());
+	_update_dirty(event_doc);
+}
+
+void REHex::MainWindow::OnTitleChanged(DocumentTitleEvent &event)
+{
+	Document *event_doc = (Document*)(event.GetEventObject());
 	
-	auto tab = dynamic_cast<Tab*>(cpage);
-	assert(tab != NULL);
-	
-	auto doc = dynamic_cast<REHex::Document*>(event.GetEventObject());
-	assert(doc != NULL);
-	
-	if(doc == tab->doc)
+	size_t num_tabs = notebook->GetPageCount();
+	for(size_t i = 0; i < num_tabs; ++i)
 	{
-		/* Only update the window if the event originated from the active document. */
-		_update_dirty(tab->doc);
+		wxWindow *page = notebook->GetPage(i);
+		assert(page != NULL);
+		
+		assert(dynamic_cast<Tab*>(page) != NULL);
+		Tab *tab = (Tab*)(page);
+		
+		if(tab->doc == event_doc)
+		{
+			notebook->SetPageText(i, tab->doc->get_title());
+			break;
+		}
 	}
+	
+	if(event_doc == active_document())
+	{
+		/* Document has a backing file (if it didn't already), enable refresh command. */
+		file_menu->Enable(wxID_REFRESH, true);
+	}
+	
+	event.Skip();
+}
+
+void REHex::MainWindow::OnFileModified(wxCommandEvent &event)
+{
+	Document *event_doc = (Document*)(event.GetEventObject());
+	_update_dirty(event_doc);
 }
 
 REHex::Tab *REHex::MainWindow::active_tab()
@@ -1966,6 +2072,22 @@ void REHex::MainWindow::switch_tab(DocumentCtrl *doc_ctrl)
 	}
 }
 
+void REHex::MainWindow::insert_tab(Tab *tab, int position)
+{
+	if(position < 0)
+	{
+		position = notebook->GetPageCount();
+	}
+	
+	tab->Reparent(notebook);
+	notebook->InsertPage(position, tab, tab->doc->get_title(), true);
+}
+
+REHex::DetachableNotebook *REHex::MainWindow::get_notebook()
+{
+	return notebook;
+}
+
 void REHex::MainWindow::_update_status_offset(Tab *tab)
 {
 	off_t off = tab->doc->get_cursor_position();
@@ -2046,19 +2168,51 @@ void REHex::MainWindow::_update_undo(REHex::Document *doc)
 
 void REHex::MainWindow::_update_dirty(REHex::Document *doc)
 {
-	bool dirty = doc->is_dirty();
-	bool has_file = (doc->get_filename() != "");
+	bool        enable_save  = doc->get_filename() == "";
+	std::string window_title = doc->get_title() + " - Reverse Engineers' Hex Editor";
+	wxBitmap    tab_bitmap   = wxNullBitmap;
 	
-	bool enable_save = dirty || !has_file;
+	if(doc->file_deleted())
+	{
+		enable_save  = true;
+		window_title = "[DELETED] " + window_title;
+		tab_bitmap   = notebook_bad_bitmap;
+	}
+	else if(doc->is_dirty() || doc->file_modified())
+	{
+		enable_save  = true;
+		window_title = "[UNSAVED] " + window_title;
+		tab_bitmap   = notebook_dirty_bitmap;
+	}
 	
-	SetTitle((dirty ? "[UNSAVED] " : "") + doc->get_title() + " - Reverse Engineers' Hex Editor");
-	
-	file_menu->Enable(wxID_SAVE,   enable_save);
-	
-	wxToolBar *toolbar = GetToolBar();
-	toolbar->EnableTool(wxID_SAVE,   enable_save);
-	
-	notebook->SetPageBitmap(notebook->GetSelection(), (dirty ? notebook_dirty_bitmap : wxNullBitmap));
+	Tab *active_tab = this->active_tab();
+	if(doc == active_tab->doc)
+	{
+		file_menu->Enable(wxID_SAVE, enable_save);
+		
+		wxToolBar *toolbar = GetToolBar();
+		toolbar->EnableTool(wxID_SAVE, enable_save);
+		
+		SetTitle(window_title);
+		notebook->SetPageBitmap(notebook->GetSelection(), tab_bitmap);
+	}
+	else{
+		size_t num_tabs = notebook->GetPageCount();
+		for(size_t i = 0; i < num_tabs; ++i)
+		{
+			wxWindow *page = notebook->GetPage(i);
+			assert(page != NULL);
+			
+			auto tab = dynamic_cast<Tab*>(page);
+			assert(tab != NULL);
+			
+			if(tab->doc == doc)
+			{
+				notebook->SetPageBitmap(i, tab_bitmap);
+				break;
+			}
+		}
+	}
 }
 
 void REHex::MainWindow::_update_cpos_buttons(DocumentCtrl *doc_ctrl)
@@ -2069,72 +2223,137 @@ void REHex::MainWindow::_update_cpos_buttons(DocumentCtrl *doc_ctrl)
 	toolbar->EnableTool(wxID_FORWARD,  doc_ctrl->has_next_cursor_position());
 }
 
-bool REHex::MainWindow::unsaved_confirm()
+bool REHex::MainWindow::confirm_close_tabs(const std::vector<Tab*> &tabs)
 {
-	std::vector<wxString> dirty_files;
+	std::vector<Tab*> deleted_tabs;
+	std::copy_if(tabs.begin(), tabs.end(), std::back_inserter(deleted_tabs),
+		[](Tab *tab) { return tab->doc->file_deleted(); });
 	
-	size_t num_tabs = notebook->GetPageCount();
-	for(size_t i = 0; i < num_tabs; ++i)
+	if(deleted_tabs.size() == 1)
 	{
-		wxWindow *page = notebook->GetPage(i);
-		assert(page != NULL);
+		Tab *tab = deleted_tabs[0];
 		
-		auto tab = dynamic_cast<Tab*>(page);
-		assert(tab != NULL);
+		wxMessageDialog confirm(
+			this,
+			(wxString("The file ") + tab->doc->get_filename() + " has been deleted from disk."),
+			"File deleted",
+			(wxYES_NO | wxCANCEL | wxCENTER));
 		
-		if(tab->doc->is_dirty())
-		{
-			dirty_files.push_back(tab->doc->get_title());
-		}
-	}
-	
-	return unsaved_confirm(dirty_files);
-}
-
-bool REHex::MainWindow::unsaved_confirm(const std::vector<wxString> &files)
-{
-	if(files.size() == 1)
-	{
-		wxMessageDialog confirm(this, (wxString("The file ") + files[0] + " has unsaved changes.\nClose anyway?"), "Unsaved changes",
-			(wxYES | wxNO | wxCENTER));
+		confirm.SetYesNoCancelLabels("Close anyway", "Save and close", "Cancel");
 		
 		int response = confirm.ShowModal();
-		
-		return response == wxID_YES;
+		switch(response)
+		{
+			case wxID_YES:
+			{
+				/* Close anyway */
+				return true;
+			}
+			
+			case wxID_NO:
+			{
+				/* Save and close */
+				
+				try {
+					tab->doc->save();
+					_update_dirty(tab->doc);
+					
+					file_menu->Enable(wxID_REFRESH, true);
+				}
+				catch(const std::exception &e)
+				{
+					wxMessageBox(
+						std::string("Error saving ") + tab->doc->get_title() + ":\n" + e.what(),
+						"Error", wxICON_ERROR, this);
+					
+					return false;
+				}
+				
+				return true;
+			}
+			
+			default:
+			{
+				/* Cancel */
+				return false;
+			}
+		}
 	}
-	else if(files.size() > 1)
+	else if(deleted_tabs.size() > 1)
 	{
-		wxString message = "The following files have unsaved changes, close anyway?\n";
+		wxString message = "The following files have been deleted from disk:\n";
 		
-		for(auto i = files.begin(); i != files.end(); ++i)
+		for(auto t = deleted_tabs.begin(); t != deleted_tabs.end(); ++t)
 		{
 			message.Append('\n');
-			message.Append(*i);
+			message.Append((*t)->doc->get_filename());
+		}
+		
+		wxMessageDialog confirm(this, message, "Files deleted",
+			(wxYES | wxNO | wxCENTER));
+		
+		confirm.SetYesNoLabels("Close anyway", "Cancel");
+		
+		int response = confirm.ShowModal();
+		return response == wxID_YES;
+	}
+	
+	std::vector<Tab*> modified_tabs;
+	std::copy_if(tabs.begin(), tabs.end(), std::back_inserter(modified_tabs),
+		[](Tab *tab) { return tab->doc->file_modified(); });
+	
+	if(modified_tabs.size() > 0)
+	{
+		wxString message = "The following files have been modified by another application:\n";
+		
+		for(auto t = modified_tabs.begin(); t != modified_tabs.end(); ++t)
+		{
+			message.Append('\n');
+			message.Append((*t)->doc->get_filename());
+		}
+		
+		wxMessageDialog confirm(this, message, "Files modified",
+			(wxYES | wxNO | wxCENTER));
+		
+		confirm.SetYesNoLabels("Close anyway", "Cancel");
+		
+		int response = confirm.ShowModal();
+		return response == wxID_YES;
+	}
+	
+	std::vector<Tab*> dirty_tabs;
+	std::copy_if(tabs.begin(), tabs.end(), std::back_inserter(dirty_tabs),
+		[](Tab *tab) { return tab->doc->is_dirty(); });
+	
+	if(dirty_tabs.size() > 0)
+	{
+		wxString message = "The following files have unsaved changes:\n";
+		
+		for(auto t = dirty_tabs.begin(); t != dirty_tabs.end(); ++t)
+		{
+			message.Append('\n');
+			message.Append((*t)->doc->get_filename());
 		}
 		
 		wxMessageDialog confirm(this, message, "Unsaved changes",
 			(wxYES | wxNO | wxCENTER));
 		
-		int response = confirm.ShowModal();
+		confirm.SetYesNoLabels("Close anyway", "Cancel");
 		
+		int response = confirm.ShowModal();
 		return response == wxID_YES;
 	}
-
+	
 	return true;
 }
 
 void REHex::MainWindow::close_tab(Tab *tab)
 {
-	if(tab->doc->is_dirty())
+	std::vector<Tab*> closing_tabs = { tab };
+	if(!confirm_close_tabs(closing_tabs))
 	{
-		std::vector<wxString> dirty_titles;
-		dirty_titles.push_back(tab->doc->get_title());
-		
-		if(!unsaved_confirm(dirty_titles))
-		{
-			/* User didn't really want to close unsaved tabs. */
-			return;
-		}
+		/* User didn't want to discard unsaved changes. */
+		return;
 	}
 	
 	notebook->DeletePage(notebook->GetPageIndex(tab));
@@ -2147,7 +2366,21 @@ void REHex::MainWindow::close_tab(Tab *tab)
 
 void REHex::MainWindow::close_all_tabs()
 {
-	if(!unsaved_confirm())
+	std::vector<Tab*> closing_tabs;
+	
+	size_t num_tabs = notebook->GetPageCount();
+	for(size_t i = 0; i < num_tabs; ++i)
+	{
+		wxWindow *page = notebook->GetPage(i);
+		assert(page != NULL);
+		
+		auto p_tab = dynamic_cast<Tab*>(page);
+		assert(p_tab != NULL);
+		
+		closing_tabs.push_back(p_tab);
+	}
+	
+	if(!confirm_close_tabs(closing_tabs))
 	{
 		/* User didn't really want to close unsaved tabs. */
 		return;
@@ -2159,7 +2392,7 @@ void REHex::MainWindow::close_all_tabs()
 
 void REHex::MainWindow::close_other_tabs(Tab *tab)
 {
-	std::vector<wxString> dirty_others;
+	std::vector<Tab*> closing_tabs;
 	
 	size_t num_tabs = notebook->GetPageCount();
 	for(size_t i = 0; i < num_tabs; ++i)
@@ -2167,21 +2400,16 @@ void REHex::MainWindow::close_other_tabs(Tab *tab)
 		wxWindow *page = notebook->GetPage(i);
 		assert(page != NULL);
 		
-		if(page == tab)
-		{
-			continue;
-		}
-		
 		auto p_tab = dynamic_cast<Tab*>(page);
 		assert(p_tab != NULL);
 		
-		if(p_tab->doc->is_dirty())
+		if(p_tab != tab)
 		{
-			dirty_others.push_back(p_tab->doc->get_title());
+			closing_tabs.push_back(p_tab);
 		}
 	}
 	
-	if(!unsaved_confirm(dirty_others))
+	if(!confirm_close_tabs(closing_tabs))
 	{
 		/* User didn't really want to close unsaved tabs. */
 		return;
