@@ -24,10 +24,12 @@
 #include <wx/filename.h>
 #include <wx/utils.h>
 
+#include "App.hpp"
 #include "CharacterEncoder.hpp"
 #include "DataType.hpp"
 #include "document.hpp"
 #include "DocumentCtrl.hpp"
+#include "SafeWindowPointer.hpp"
 #include "util.hpp"
 
 /* These MUST come after any wxWidgets headers. */
@@ -347,3 +349,127 @@ void REHex::copy_from_doc(REHex::Document *doc, REHex::DocumentCtrl *doc_ctrl, w
 		}
 	}
 }
+
+class GenuineImmitationMouseCapture
+{
+	public:
+		GenuineImmitationMouseCapture(wxWindow *window);
+	
+	private:
+		REHex::SafeWindowPointer<wxWindow> window;
+		wxTimer timer;
+		bool left_was_down;
+		wxPoint last_mouse_pos;
+		
+		void OnTimer(wxTimerEvent &event);
+		void OnWindowDestroy(wxWindowDestroyEvent &event);
+};
+
+GenuineImmitationMouseCapture::GenuineImmitationMouseCapture(wxWindow *window):
+	window(window)
+{
+	left_was_down = wxGetMouseState().LeftIsDown();
+	last_mouse_pos = wxGetMousePosition();
+	
+	timer.Bind(wxEVT_TIMER, &GenuineImmitationMouseCapture::OnTimer, this);
+	this->window.auto_cleanup_bind(wxEVT_DESTROY, &GenuineImmitationMouseCapture::OnWindowDestroy, this);
+	
+	timer.Start(50, wxTIMER_CONTINUOUS);
+}
+
+void GenuineImmitationMouseCapture::OnTimer(wxTimerEvent &event)
+{
+	if(!window->HasCapture()) /* Detect if the window has called ReleaseMouse() */
+	{
+		timer.Stop();
+		
+		window->CallAfter([=]()
+		{
+			/* Destroying the timer in its event handler would probably do bad things. */
+			delete this;
+		});
+		
+		return;
+	}
+	
+	wxMouseState mouse_state = wxGetMouseState();
+	wxPoint mouse_pos = wxGetMousePosition();
+	
+	if(mouse_pos != last_mouse_pos)
+	{
+		last_mouse_pos = mouse_pos;
+		
+		wxMouseEvent e(wxEVT_MOTION);
+		window->ProcessWindowEvent(e);
+	}
+	
+	if(left_was_down && !mouse_state.LeftIsDown())
+	{
+		left_was_down = false;
+		
+		wxMouseEvent e(wxEVT_LEFT_UP);
+		window->ProcessWindowEvent(e);
+	}
+	else if(!left_was_down && mouse_state.LeftIsDown())
+	{
+		left_was_down = true;
+		
+		wxMouseEvent e(wxEVT_LEFT_DOWN);
+		window->ProcessWindowEvent(e);
+	}
+}
+
+void GenuineImmitationMouseCapture::OnWindowDestroy(wxWindowDestroyEvent &event)
+{
+	if(event.GetEventObject() == window)
+	{
+		delete this;
+	}
+	
+	event.Skip();
+}
+
+void REHex::fake_broken_mouse_capture(wxWindow *window)
+{
+	if(!window->HasCapture())
+	{
+		/* Window isn't trying to capture the mouse... probably? */
+		return;
+	}
+	
+	new GenuineImmitationMouseCapture(window);
+}
+
+std::string REHex::document_save_as_dialog(wxWindow *modal_parent, Document *document)
+{
+	std::string dir, name;
+	std::string doc_filename = document->get_filename();
+	
+	if(doc_filename != "")
+	{
+		wxFileName wxfn(doc_filename);
+		wxfn.MakeAbsolute();
+		
+		dir  = wxfn.GetPath();
+		name = wxfn.GetFullName();
+	}
+	else{
+		dir  = wxGetApp().get_last_directory();
+		name = "";
+	}
+	
+	wxFileDialog saveFileDialog(modal_parent, "Save As", dir, name, "", wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
+	if(saveFileDialog.ShowModal() == wxID_CANCEL)
+		return "";
+	
+	std::string filename = saveFileDialog.GetPath().ToStdString();
+	
+	{
+		wxFileName wxfn(filename);
+		wxString dirname = wxfn.GetPath();
+		
+		wxGetApp().set_last_directory(dirname.ToStdString());
+	}
+	
+	return filename;
+}
