@@ -89,7 +89,7 @@ class FixedHeightRegion: public DocumentCtrl::Region
 			Region(indent_offset, indent_length),
 			height(height) {}
 		
-		virtual void calc_height(DocumentCtrl &doc, wxDC &dc) override
+		virtual void calc_height(DocumentCtrl &doc) override
 		{
 			y_lines = height + indent_final;
 		}
@@ -113,7 +113,7 @@ class FixedHeightDataRegion: public DocumentCtrl::GenericDataRegion
 			GenericDataRegion(d_offset, d_length, d_offset, indent_offset),
 			height(height) {}
 		
-		virtual void calc_height(DocumentCtrl &doc, wxDC &dc) override
+		virtual void calc_height(DocumentCtrl &doc) override
 		{
 			y_lines = height + indent_final;
 		}
@@ -1741,10 +1741,10 @@ TEST_F(DocumentCtrlTest, RegionOffsetCompare)
 	doc->insert_data(0, Z_DATA.data(), Z_DATA.size());
 	
 	std::vector<DocumentCtrl::Region*> regions = {
-		new DocumentCtrl::DataRegion(doc, 200, 50, 200),
-		new DocumentCtrl::DataRegion(doc, 150, 50, 250),
-		new DocumentCtrl::DataRegion(doc, 300, 50, 300),
-		new DocumentCtrl::DataRegion(doc, 350, 50, 350),
+		new DocumentCtrl::DataRegion(doc, 200, 50,  200),
+		new DocumentCtrl::DataRegion(doc, 150, 50,  250),
+		new DocumentCtrl::DataRegion(doc, 300, 50,  300),
+		new DocumentCtrl::DataRegion(doc, 350, 100, 350),
 	};
 	
 	doc_ctrl->replace_all_regions(regions);
@@ -1752,13 +1752,21 @@ TEST_F(DocumentCtrlTest, RegionOffsetCompare)
 	EXPECT_EQ(doc_ctrl->region_offset_cmp(150, 150), 0);
 	EXPECT_EQ(doc_ctrl->region_offset_cmp(350, 350), 0);
 	
-	EXPECT_LT(doc_ctrl->region_offset_cmp(150, 180), 0);
-	EXPECT_LT(doc_ctrl->region_offset_cmp(200, 150), 0);
-	EXPECT_LT(doc_ctrl->region_offset_cmp(300, 350), 0);
+	EXPECT_EQ(doc_ctrl->region_offset_cmp(150, 180), -30);
+	EXPECT_EQ(doc_ctrl->region_offset_cmp(200, 150), -50);
+	EXPECT_EQ(doc_ctrl->region_offset_cmp(300, 350), -50);
+	EXPECT_EQ(doc_ctrl->region_offset_cmp(200, 350), -150);
+	EXPECT_EQ(doc_ctrl->region_offset_cmp(200, 399), -199);
+	EXPECT_EQ(doc_ctrl->region_offset_cmp(220, 350), -130);
+	EXPECT_EQ(doc_ctrl->region_offset_cmp(220, 390), -170);
 	
-	EXPECT_GT(doc_ctrl->region_offset_cmp(180, 150), 0);
-	EXPECT_GT(doc_ctrl->region_offset_cmp(150, 200), 0);
-	EXPECT_GT(doc_ctrl->region_offset_cmp(350, 300), 0);
+	EXPECT_EQ(doc_ctrl->region_offset_cmp(180, 150), 30);
+	EXPECT_EQ(doc_ctrl->region_offset_cmp(150, 200), 50);
+	EXPECT_EQ(doc_ctrl->region_offset_cmp(350, 300), 50);
+	EXPECT_EQ(doc_ctrl->region_offset_cmp(350, 200), 150);
+	EXPECT_EQ(doc_ctrl->region_offset_cmp(399, 200), 199);
+	EXPECT_EQ(doc_ctrl->region_offset_cmp(350, 220), 130);
+	EXPECT_EQ(doc_ctrl->region_offset_cmp(390, 220), 170);
 	
 	EXPECT_THROW(doc_ctrl->region_offset_cmp(100, 150), std::invalid_argument);
 	EXPECT_THROW(doc_ctrl->region_offset_cmp(150, 100), std::invalid_argument);
