@@ -50,14 +50,17 @@ int* p_wxluatype_wxEvent             = &wxluatype_TUNKNOWN;
 int* p_wxluatype_wxWindow            = &wxluatype_TUNKNOWN;
 int* p_wxluatype_wxScrollEvent       = &wxluatype_TUNKNOWN;
 int* p_wxluatype_wxSpinEvent         = &wxluatype_TUNKNOWN;
+int* p_wxluatype_wxUniChar           = &wxluatype_TUNKNOWN;
 int* p_wxluatype_wxString            = &wxluatype_TUNKNOWN;
 int* p_wxluatype_wxArrayString       = &wxluatype_TUNKNOWN;
 int* p_wxluatype_wxSortedArrayString = &wxluatype_TUNKNOWN;
 int* p_wxluatype_wxArrayInt          = &wxluatype_TUNKNOWN;
 int* p_wxluatype_wxArrayDouble       = &wxluatype_TUNKNOWN;
 int* p_wxluatype_wxMemoryBuffer      = &wxluatype_TUNKNOWN;
+int* p_wxluatype_wxDateTime          = &wxluatype_TUNKNOWN;
 int* p_wxluatype_wxPoint             = &wxluatype_TUNKNOWN;
 int* p_wxluatype_wxPoint2DDouble     = &wxluatype_TUNKNOWN;
+int* p_wxluatype_wxVariant           = &wxluatype_TUNKNOWN;
 
 // ----------------------------------------------------------------------------
 // wxlua_tableErrorHandler
